import { ManagedSocketErrorType, NON_RECONNECT_CODES } from "./constants";
import { ManagedSocketError } from "./errors";
import {
  type SerializedSocketMessage,
  type SocketDataMessage,
  type SocketHeartbeatMessage,
  type SocketMessage,
  parseAnyScoketMessage,
} from "./types";

/**
 * Creates a socket data message object with the given data.
 *
 * @param data - The data to be included in the message. Can be of any type.
 * @returns An object representing the socket data message with a type of "message" and the provided data.
 */
function createDataMessage(data: unknown): SocketDataMessage {
  return {
    type: "message",
    data,
  };
}

/**
 * Creates a heartbeat message to be sent over a socket connection.
 *
 * @returns {SocketHeartbeatMessage} An object representing a heartbeat message with the current timestamp.
 */
function createHeartbeatMessage(): SocketHeartbeatMessage {
  return {
    type: "heartbeat",
    data: new Date().toISOString(),
  };
}

/**
 * Serializes a SocketMessage object into a JSON string.
 *
 * @param {SocketMessage} message - The message object to serialize.
 * @returns {SerializedSocketMessage} The serialized JSON string representation of the message.
 */
function serialize(message: SocketMessage): SerializedSocketMessage {
  return JSON.stringify(message) as SerializedSocketMessage;
}

/**
 * Deserializes a JSON string into a `SocketMessage` object.
 *
 * @param data - The JSON string to be deserialized.
 * @returns The deserialized `SocketMessage` object.
 * @throws {ManagedSocketError} If the JSON string cannot be parsed or if the message shape is invalid.
 */
function deserialize(data: string): SocketMessage {
  try {
    const asObj = JSON.parse(data);
    return parseAnyScoketMessage(asObj);
  } catch (error) {
    throw new ManagedSocketError(
      "Failed to parse incoming message shape",
      error instanceof Error ? error : new Error("Unknown error"),
      ManagedSocketErrorType.INVALID_MESSAGE_SHAPE,
    );
  }
}

/**
 * Creates and serializes a socket message with the given data.
 *
 * @remarks This is a convenience method to streamline making
 * data messages. It's the equivalent of:
 * `serialize(createDataMessage(data))`
 *
 * @template T - The type of the data to be prepared.
 * @param {T} data - The data to be included in the message.
 * @returns {SerializedSocketMessage} - The serialized socket message.
 */
function prepare<T>(data: T): SerializedSocketMessage {
  return serialize(createDataMessage(data));
}

/**
 * Determines if the provided close code is considered catastrophic.
 *
 * @param code - The close code to check.
 * @returns `true` if the code is in the list of non-reconnect codes, otherwise `false`.
 */
function isCatastrophicCloseCode(code: number) {
  return (NON_RECONNECT_CODES as readonly number[]).includes(code);
}

export const SocketSpeak = {
  createDataMessage,
  createHeartbeatMessage,
  serialize,
  deserialize,
  isCatastrophicCloseCode,
  prepare,
};
