import { test, describe, expect } from "bun:test";

import { parseAnyScoketMessage, parseMessageType } from "../types";
import { ManagedSocketError } from "../errors";

describe("parseMessageType()", () => {
  test("errors if the value is not a string", () => {
    expect(() => parseMessageType(123)).toThrowError(ManagedSocketError);
  });

  test("errors if the value is a string but not a valid message type", () => {
    expect(() => parseMessageType("invalid")).toThrowError(ManagedSocketError);
  });

  test("returns the message type when the value is a valid message type", () => {
    expect(parseMessageType("heartbeat")).toBe("heartbeat");
    expect(parseMessageType("message")).toBe("message");
  });
});

describe("parseAnyScoketMessage()", () => {
  test("throws when the value is null", () => {
    expect(() => parseAnyScoketMessage(null)).toThrowError(ManagedSocketError);
  });

  test("throws when the value is not an object", () => {
    expect(() => parseAnyScoketMessage("string")).toThrowError(
      ManagedSocketError,
    );
    expect(() => parseAnyScoketMessage(() => {})).toThrowError(
      ManagedSocketError,
    );
    expect(() => parseAnyScoketMessage(false)).toThrowError(ManagedSocketError);
    expect(() => parseAnyScoketMessage([])).toThrowError(ManagedSocketError);
    expect(() => parseAnyScoketMessage(Symbol())).toThrowError(
      ManagedSocketError,
    );
  });

  test("throws when the object does not have the correct shape", () => {
    expect(() => parseAnyScoketMessage({})).toThrowError(ManagedSocketError);
    expect(() => parseAnyScoketMessage({ type: "message" })).toThrowError(
      ManagedSocketError,
    );
    expect(() => parseAnyScoketMessage({ data: "data" })).toThrowError(
      ManagedSocketError,
    );
  });

  test("throws when the message type is invalid", () => {
    expect(() =>
      parseAnyScoketMessage({ type: "invalid", data: "data" }),
    ).toThrowError(ManagedSocketError);
  });

  test("throws when the type is 'heartbeat' and the value is not a string", () => {
    expect(() =>
      parseAnyScoketMessage({ type: "heartbeat", data: 123 }),
    ).toThrowError(ManagedSocketError);
  });

  test("returns a valid message object when the type is 'heartbeat'", () => {
    const dateStr = new Date().toISOString();
    const message = parseAnyScoketMessage({
      type: "heartbeat",
      data: dateStr,
    });
    const dataMessage = parseAnyScoketMessage({ type: "message", data: 123 });
    expect(message).toEqual({ type: "heartbeat", data: dateStr });
    expect(dataMessage).toEqual({ type: "message", data: 123 });
  });
});
